@@ -13,8 +13,6 @@ module Rerun
     end
 
     def command
-      return unless mac?
-
       # todo: strategy or subclass
 
       s = nil
@@ -34,17 +32,28 @@ module Rerun
         end
       end
 
+      if s.nil? and options[:notify] == true or options[:notify] == "notify-send"
+        if (cmd = command_named('notify-send'))
+          icon_str = "--icon #{icon}" if icon
+          s = "#{cmd} -t 500 --hint=int:transient:1 #{icon_str} \"#{app_name}: #{title}\" \"#{body}\""
+        end
+      end
+
       s
     end
 
     def command_named(name)
-      path = `which #{name}`.chomp
+      which_command = windows? ? 'where.exe' : 'which'
+      # TODO: remove 'INFO' error message
+      path = `#{which_command} #{name}`.chomp
       path.empty? ? nil : path
     end
 
     def send(background = true)
       return unless command
-      `#{command}#{" &" if background}`
+      with_clean_env do
+        `#{command}#{" &" if background}`
+      end
     end
 
     def app_name
@@ -60,5 +69,14 @@ module Rerun
       File.expand_path("#{here}/../../icons")
     end
 
+    def with_clean_env
+      if defined?(Bundler)
+        Bundler.with_clean_env do
+          yield
+        end
+      else
+        yield
+      end
+    end
   end
 end
