@@ -9,10 +9,11 @@ module Rerun
       runner.start
       runner.join
       # apparently runner doesn't keep running anymore (as of Listen 2) so we have to sleep forever :-(
-      sleep 10000 while true  # :-(
+      sleep 10000 while true # :-(
     end
 
     include System
+    include ::Timeout
 
     def initialize(run_command, options = {})
       @run_command, @options = run_command, options
@@ -32,22 +33,26 @@ module Rerun
             when 'r'
               say "Restarting"
               restart
+            when 'f'
+              say "Stopping and starting"
+              restart(false)
             when 'p'
-              toggle_pause if watcher_running?
+              toggle_pause
             when 'x', 'q'
               die
-              break  # the break will stop this thread, in case the 'die' doesn't
+              break # the break will stop this thread, in case the 'die' doesn't
             else
               puts "\n#{c.inspect} pressed inside rerun"
               puts [["c", "clear screen"],
-               ["r", "restart"],
-               ["p", "toggle pause"],
-               ["x or q", "stop and exit"]
-              ].map{|key, description| "  #{key} -- #{description}"}.join("\n")
+                    ["r", "restart"],
+                    ["f", "forced restart (stop and start)"],
+                    ["p", "toggle pause"],
+                    ["x or q", "stop and exit"]
+                   ].map {|key, description| "  #{key} -- #{description}"}.join("\n")
               puts
             end
           end
-          sleep 1  # todo: use select instead of polling somehow?
+          sleep 1 # todo: use select instead of polling somehow?
         end
       end
       @keypress_thread.run
@@ -58,9 +63,9 @@ module Rerun
       @keypress_thread = nil
     end
 
-    def restart
+    def restart(with_signal = true)
       @restarting = true
-      if @options[:restart]
+      if @options[:restart] && with_signal
         restart_with_signal(@options[:signal])
       else
         stop
@@ -69,10 +74,6 @@ module Rerun
       @restarting = false
     end
 
-    def watcher_running?
-      @watcher && @watcher.running?
-    end
-
     def toggle_pause
       unless @pausing
         say "Pausing.  Press 'p' again to resume."
@@ -109,6 +110,14 @@ module Rerun
       @options[:clear]
     end
 
+    def quiet?
+      @options[:quiet]
+    end
+
+    def verbose?
+      @options[:verbose]
+    end
+
     def exit?
       @options[:exit]
     end
@@ -120,15 +129,15 @@ module Rerun
     def restart_with_signal(restart_signal)
       if @pid && (@pid != 0)
         notify "restarting", "We will be with you shortly."
-        signal(restart_signal)
+        send_signal(restart_signal)
       end
     end
 
-    def start
-      if windows?
-        raise "Sorry, Rerun does not work on Windows."
-      end
+    def force_polling
+      @options[:force_polling]
+    end
 
+    def start
       if @already_running
         taglines = [
           "Here we go again!",
@@ -149,21 +158,21 @@ module Rerun
       clear_screen if clear?
       start_keypress_thread unless @keypress_thread
 
-      @pid = Kernel.fork do
-        begin
-          exec(@run_command)
-        rescue => e
-          puts "#{e.class}: #{e.message}"
-          exit
-        end
+      begin
+        @pid = run @run_command
+        say "Rerun (#{$PID}) running #{app_name} (#{@pid})"
+      rescue => e
+        puts "#{e.class}: #{e.message}"
+        exit
       end
+
       status_thread = Process.detach(@pid) # so if the child exits, it dies
 
       Signal.trap("INT") do # INT = control-C -- allows user to stop the top-level rerun process
         die
       end
 
-      Signal.trap("TERM") do  # TERM is the polite way of terminating a process
+      Signal.trap("TERM") do # TERM is the polite way of terminating a process
         die
       end
 
@@ -190,7 +199,7 @@ module Rerun
 
       unless @watcher
 
-        watcher = Watcher.new(:directory => dirs, :pattern => pattern, :ignore => ignore) do |changes|
+        watcher = Watcher.new(:directory => dirs, :pattern => pattern, :ignore => ignore, :force_polling => force_polling) do |changes|
 
           message = change_message(changes)
 
@@ -200,10 +209,15 @@ module Rerun
         watcher.start
         @watcher = watcher
         say "Watching #{dir.join(', ')} for #{pattern}" +
-                (ignore.empty? ? "" : " (ignoring #{ignore.join(',')})")
+              (ignore.empty? ? "" : " (ignoring #{ignore.join(',')})") +
+              (watcher.adapter.nil? ? "" : " with #{watcher.adapter_name} adapter")
       end
     end
 
+    def run command
+      Kernel.spawn command
+    end
+
     def change_message(changes)
       message = [:modified, :added, :removed].map do |change|
         count = changes[change] ? changes[change].size : 0
@@ -215,7 +229,7 @@ module Rerun
       changed_files = changes.values.flatten
       if changed_files.count > 0
         message += ": "
-        message += changes.values.flatten[0..3].map { |path| path.split('/').last }.join(', ')
+        message += changes.values.flatten[0..3].map {|path| path.split('/').last}.join(', ')
         if changed_files.count > 3
           message += ", ..."
         end
@@ -226,7 +240,7 @@ module Rerun
     def die
       #stop_keypress_thread   # don't do this since we're probably *in* the keypress thread
       stop # stop the child process if it exists
-      exit 0  # todo: status code param
+      exit 0 # todo: status code param
     end
 
     def join
@@ -234,37 +248,60 @@ module Rerun
     end
 
     def running?
-      signal(0)
+      send_signal(0)
     end
 
-    def signal(signal)
-      say "Sending signal #{signal} to #{@pid}" unless signal == 0
+    # Send the signal to process @pid and wait for it to die.
+    # @returns true if the process dies
+    # @returns false if either sending the signal fails or the process fails to die
+    def signal_and_wait(signal)
+
+      signal_sent = if windows?
+                      force_kill = (signal == 'KILL')
+                      system("taskkill /T #{'/F' if force_kill} /PID #{@pid}")
+                    else
+                      send_signal(signal)
+                    end
+
+      if signal_sent
+        # the signal was successfully sent, so wait for the process to die
+        begin
+          timeout(@options[:wait]) do
+            Process.wait(@pid)
+          end
+          process_status = $?
+          say "Process ended: #{process_status}" if verbose?
+          true
+        rescue Timeout::Error
+          false
+        end
+      else
+        false
+      end
+    end
+
+    # Send the signal to process @pid.
+    # @returns true if the signal is sent
+    # @returns false if sending the signal fails
+    # If sending the signal fails, the exception will be swallowed
+    # (and logged if verbose is true) and this method will return false.
+    #
+    def send_signal(signal)
+      say "Sending signal #{signal} to #{@pid}" unless signal == 0 if verbose?
       Process.kill(signal, @pid)
       true
-    rescue
+    rescue => e
+      say "Signal #{signal} failed: #{e.class}: #{e.message}" if verbose?
       false
     end
 
     # todo: test escalation
     def stop
-      default_signal = @options[:signal] || "TERM"
       if @pid && (@pid != 0)
         notify "stopping", "All good things must come to an end." unless @restarting
-        begin
-          timeout(5) do  # todo: escalation timeout setting
-            # start with a polite SIGTERM
-            signal(default_signal) && Process.wait(@pid)
-          end
-        rescue Timeout::Error
-          begin
-            timeout(5) do
-              # escalate to SIGINT aka control-C since some foolish process may be ignoring SIGTERM
-              signal("INT") && Process.wait(@pid)
-            end
-          rescue Timeout::Error
-            # escalate to SIGKILL aka "kill -9" which cannot be ignored
-            signal("KILL") && Process.wait(@pid)
-          end
+        @options[:signal].split(',').each do |signal|
+          success = signal_and_wait(signal)
+          return true if success
         end
       end
     rescue => e
@@ -289,7 +326,11 @@ module Rerun
     end
 
     def say msg
-      puts "#{Time.now.strftime("%T")} [rerun] #{msg}"
+      puts "#{Time.now.strftime("%T")} [rerun] #{msg}" unless quiet?
+    end
+
+    def stty(args)
+      system "stty #{args}"
     end
 
     # non-blocking stdin reader.
@@ -305,7 +346,7 @@ module Rerun
         # looks like "raw" flips off the OPOST bit 0x00000001 /* enable following output processing */
         # which disables #define ONLCR		0x00000002	/* map NL to CR-NL (ala CRMOD) */
         # so this sets it back on again since all we care about is raw input, not raw output
-        system("stty raw opost")
+        stty "raw opost"
 
         c = nil
         if $stdin.ready?
@@ -313,9 +354,10 @@ module Rerun
         end
         c.chr if c
       ensure
-        system "stty -raw" # turn raw input off
+        stty "-raw" # turn raw input off
       end
 
+
       # note: according to 'man tty' the proper way restore the settings is
       # tty_state=`stty -g`
       # ensure
