@@ -39,6 +39,7 @@ module Rerun
       @directories = options[:directory]
       @directories = sanitize_dirs(@directories)
       @priority = options[:priority]
+      @force_polling = options[:force_polling]
       @ignore = [options[:ignore]].flatten.compact
       @thread = nil
     end
@@ -60,8 +61,9 @@ module Rerun
       end
 
       @thread = Thread.new do
-        @listener = Listen.to(*@directories, only: watching, ignore: ignoring, wait_for_delay: 1) do |modified, added, removed|
-          if((modified.size + added.size + removed.size) > 0)
+        @listener = Listen.to(*@directories, only: watching, ignore: ignoring, wait_for_delay: 1, force_polling: @force_polling) do |modified, added, removed|
+          count = modified.size + added.size + removed.size
+          if count > 0
             @client_callback.call(:modified => modified, :added => added, :removed => removed)
           end
         end
@@ -115,5 +117,15 @@ module Rerun
       @listener && @listener.processing?
     end
 
+    def adapter
+      @listener &&
+          (backend = @listener.instance_variable_get(:@backend)) &&
+          backend.instance_variable_get(:@adapter)
+    end
+
+    def adapter_name
+      adapter && adapter.class.name.split('::').last
+    end
+
   end
 end
