@@ -25,7 +25,6 @@ module Rerun
       end
     end
 
-
     before do
       @dir = Dir.tmpdir + "/#{Time.now.to_i}-#{(rand*100000).to_i}"
       # fix goofy MacOS /tmp path ambiguity
@@ -140,13 +139,24 @@ module Rerun
       @watcher.pause
       create "#{@dir}/pause_test.txt"
       @log.should be_nil
-      sleep 0.25 # work around very rare timing issue on Travis
+      sleep 0.5 # work around rare timing issue on Travis
       @watcher.unpause
       test_file = "#{@dir}/pause_test2.txt"
       create test_file
       @log[:added].should == [test_file]
     end
 
+    it "can figure out the Listen adapter" do
+      @watcher.adapter.class.should == Listen::Adapter.select
+    end
+
+    it "passes the force-polling option to Listen" do
+      stop_watcher
+      start_watcher force_polling: true
+      @watcher.adapter.class.should == Listen::Adapter::Polling
+      @watcher.adapter_name.should == "Polling"
+    end
+
   end
 
 end
